@@ -2,7 +2,7 @@
 """
 import logging
 
-log=logging.getLogger()
+log=logging.getLogger('migrate.versioning')
 log.setLevel(logging.WARNING)
 log.addHandler(logging.StreamHandler())
 
