@@ -1,8 +1,10 @@
 from sqlalchemy import Table,Column,MetaData,String,Text,Integer,create_engine
+from sqlalchemy.sql import and_
 from sqlalchemy import exceptions as sa_exceptions
 from migrate.versioning.repository import Repository
+from migrate.versioning.util import loadModel
 from migrate.versioning.version import VerNum
-from migrate.versioning import exceptions
+from migrate.versioning import exceptions, genmodel, schemadiff
 
 class ControlledSchema(object):
     """A database under version control"""
@@ -31,7 +33,9 @@ class ControlledSchema(object):
             except (exceptions.NoSuchTableError):
                 raise exceptions.DatabaseNotControlledError(tname)
         # TODO?: verify that the table is correct (# cols, etc.)
-        result = self.engine.execute(self.table.select(),)
+        result = self.engine.execute(self.table.select(
+                    self.table.c.repository_id == str(self.repository.id))
+                )
         data = list(result)[0]
         # TODO?: exception if row count is bad
         # TODO: check repository id, exception if incorrect
@@ -85,12 +89,47 @@ class ControlledSchema(object):
             )
             table.create()
         except (sa_exceptions.ArgumentError,sa_exceptions.SQLError):
-            # The table already exists
-            raise exceptions.DatabaseAlreadyControlledError()
+            # The table already exists, skip creation.
+            pass
+            #raise exceptions.DatabaseAlreadyControlledError()
         # Insert data
-        engine.execute(table.insert(),repository_id=repository.id,
-            repository_path=repository.path,version=int(version))
+        try:
+            engine.execute(table.insert(),repository_id=repository.id,
+                repository_path=repository.path,version=int(version))
+        except sa_exceptions.IntegrityError:
+            # An Entry for this repo already exists.
+            raise exceptions.DatabaseAlreadyControlledError()
         return table
+    
+    @classmethod
+    def compare_model_to_db(cls,engine,model,repository):
+        """Compare the current model against the current database."""
+
+        if isinstance(repository, basestring):
+            repository=Repository(repository)
+        model = loadModel(model)
+        diff = schemadiff.getDiffOfModelAgainstDatabase(model, engine, excludeTables=[repository.version_table])
+        return diff
+
+    @classmethod
+    def create_model(cls,engine,repository):
+        """Dump the current database as a Python model."""
+
+        if isinstance(repository, basestring):
+            repository=Repository(repository)
+        diff = schemadiff.getDiffOfModelAgainstDatabase(MetaData(), engine, excludeTables=[repository.version_table])
+        return genmodel.ModelGenerator(diff).toPython()
+    
+    def update_db_from_model(self,model):
+        """Modify the database to match the structure of the current Python model."""
+
+        if isinstance(self.repository, basestring):
+            self.repository=Repository(self.repository)
+        model = loadModel(model)
+        diff = schemadiff.getDiffOfModelAgainstDatabase(model, self.engine, excludeTables=[self.repository.version_table])
+        genmodel.ModelGenerator(diff).applyModel()
+        update = self.table.update(self.table.c.repository_id == str(self.repository.id))
+        self.engine.execute(update, version=int(self.repository.latest))
 
     def drop(self):
         """Remove version control from a database"""
@@ -119,7 +158,8 @@ class ControlledSchema(object):
         # Run the change
         change.run(self.engine,step)
         # Update/refresh database version
-        update = self.table.update(self.table.c.version == int(startver))
+        update = self.table.update(and_(self.table.c.version == int(startver),
+                                   self.table.c.repository_id == str(self.repository.id)))
         self.engine.execute(update, version=int(endver))
         self._load()
         
